//====================================================================================
// medWAHUD
// creator:	Helen! www.birdieman.com/forum
// created:	2010.08
// comment: displays the damage scaling provided by the class medWeaponAmplifier
//====================================================================================
class medWAHUD expands Mutator;

struct AmmoInfo
{
    var string DamageType;
    var string Percentage;
};

// class vars
var bool bCanRender;
var bool bOneTimePrep;
var bool bGameOn;

var Texture mBorderTexture;
var Texture mHeaderTexture;
var Texture mInfoTexture;
var Texture mFooterTexture;

var font mHeaderFont;
var font mInfoFont;
var font mFooterFont;
var int mScaledTotal;

var float chainsaw;
var float hammer;
var float pulse;
var float pulseAlt;
var float flakripper;
var float flakAlt;
var float ripperAlt;
var float rocket;
var float rocketAlt;
var float shock;
var float bio;
var float redeemer;
var float bullet;
var float selfInflicted;
var float falling;

var AmmoInfo mAmmoInfo[15];

//var() PlayerPawn mPlayerPawn;


replication
{
    // Send these values from server to client
    reliable if (Role == ROLE_Authority)
        chainsaw,
        hammer,
        pulse,
        pulseAlt,
        flakripper,
        flakAlt,
        ripperAlt,
        rocket,
        rocketAlt,
        shock,
        bio,
        redeemer,
        bullet,
        selfInflicted,
        falling,
        bGameOn,
        bCanRender;
}

simulated function Tick(float DeltaTime)
{
	if( Level.NetMode == NM_DedicatedServer )
    {
        // No need to run on server
		Disable('Tick');
		return;
    }

    if (!bHUDMutator)
    {
        RegisterHUDMutator();
        SetTimer(0.5, false);
    }

	Disable('Tick');
}

simulated function Timer()
{
    if (!bHUDMutator)
    {
        RegisterHUDMutator();
        SetTimer(0.5, false);
    }
}

simulated function PostRender(Canvas c)
{
    local int i;
    local int x;
    local int y;
    local int xText;
    local int xPerc;
    local int yText;
    local int nLeftEdge;
    local int nTopEdge;
    local int nWidth;
    local int nHeight;
    local int nBorderBreadth;
    local int nMainBoxWidth;
    local int nInfoBoxWidth;
    local int nInfoBoxHeight;
    local int nHeaderBoxHeight;
    local int nTotalHeight;
    local int nFontSlot;
    local int nSplitterAdd;
    local bool bContinue;
    local string aViewType;
    local string sNote;
    local texture aTex;
    local PlayerPawn aPlayerPawn;
    
    // I am not even sure if a postRender() call would happen for a dedicated server, but no harm.
	if( Level.NetMode == NM_DedicatedServer )
        return;

    //  bCanRender  - This gets set to true by the medWeaponAmplifier class when it has finished setting up this class.
    //  bHUDMutator - Tick() or SetTimer() is trying to set this to true.
    //  bGameOn     - This gets set to true by the medWeaponAmplifier class when it determines the game has started.
    if(!bCanRender || !bHUDMutator || bGameOn)
        return;

    aPlayerPawn = c.Viewport.Actor;

    // If no player looking at canvas, nothing to do.
    if(aPlayerPawn == None)
        return;

    // Only draw this for a TournamentPlayer
    if(!aPlayerPawn.IsA('TournamentPlayer'))
        return;

    if(mBorderTexture == None)
        mBorderTexture = Texture(DynamicLoadObject("medPix100.pixel.BLACKPIX", class'Texture'));

    if(mHeaderTexture == None)
        mHeaderTexture = Texture(DynamicLoadObject("medPix100.pixel.redpix", class'Texture'));

    if(mInfoTexture == None)
        mInfoTexture = Texture(DynamicLoadObject("medPix100.pixel.GREENPIX", class'Texture'));

    if(mFooterTexture == None)
        mFooterTexture = Texture(DynamicLoadObject("medPix100.pixel.BLUEPIX", class'Texture'));

    if(mHeaderFont == None)
        mHeaderFont = Font(DynamicLoadObject("LadderFonts.UTLadder16", class'Font'));

    if(mInfoFont == None)
        mInfoFont = Font(DynamicLoadObject("LadderFonts.UTLadder16", class'Font'));

    if(mFooterFont == None)
        mFooterFont = Font(DynamicLoadObject("LadderFonts.UTLadder10", class'Font'));

    if(!bOneTimePrep)
    {
        mScaledTotal = 0;

        ProcessAmmo(redeemer,       "Redeemer");
        ProcessAmmo(shock,          "Shock Rifle");
        ProcessAmmo(rocket,         "Rocket Launcher primary fire");
        ProcessAmmo(rocketAlt,      "Rocket Launcher secondary fire");
        ProcessAmmo(bullet,         "Bullets (Minigun, Sniper, Enforcer)");
        ProcessAmmo(pulse,          "Pulse Gun primary fire");
        ProcessAmmo(pulseAlt,       "Pulse Gun secondary fire");
        ProcessAmmo(flakripper,     "Flak and Ripper primary fire");
        ProcessAmmo(flakAlt,        "Flak secondary fire");
        ProcessAmmo(ripperAlt,      "Ripper secondary fire");
        ProcessAmmo(bio,            "Bio Gun");
        ProcessAmmo(hammer,         "Hammer");
        ProcessAmmo(chainsaw,       "Chainsaw");
        ProcessAmmo(selfInflicted,  "Self-Inflicted damage");
        ProcessAmmo(falling,        "Falling too far");

        bOneTimePrep = true;
    }

    // Nothing scaled, so don't bother with the display.
    if(mScaledTotal == 0)
        return;

    if(mScaledTotal == 1)
        sNote = "This type of damage will be increased by the above percentage";
    else
        sNote = "These types of damage will be increased by the above percentages";

    nMainBoxWidth = 384;
    nBorderBreadth = 5;
    nFontSlot = 24;
    nHeaderBoxHeight = 20;
    nSplitterAdd = 15;

    nInfoBoxWidth = nMainBoxWidth - (nBorderBreadth * 2);

    nLeftEdge = (c.ClipX/2) - (nMainBoxWidth/2);
    nTopEdge = (c.ClipY/2) - 128;

    x = nLeftEdge;
    y = nTopEdge;

    xText = nLeftEdge + nBorderBreadth + 32;
    xPerc = nLeftEdge + nBorderBreadth + ((nInfoBoxWidth * 2)/3);

    // top border
    x += nBorderBreadth;
    DrawBorder(c, x, y, nInfoBoxWidth, nBorderBreadth);

    // header box
    y += nBorderBreadth;
    DrawInfoBox(c, mHeaderTexture, x, y, nInfoBoxWidth, nHeaderBoxHeight);

    // header splitter
    DrawBorder(c, xPerc - (nBorderBreadth*2) + nSplitterAdd, y, nBorderBreadth, nHeaderBoxHeight);

    // header text
    yText = y + (nBorderBreadth/2);
    DoDrawText(c, mHeaderFont, xText, yText, 245, 255, 250, "Ammo");
    DoDrawText(c, mHeaderFont, xPerc + nSplitterAdd + 8, yText, 245, 255, 250, "Increase");

    // header bottom border
    y += nHeaderBoxHeight;
    DrawBorder(c, x, y, nInfoBoxWidth, nBorderBreadth);

    // info box
    y += nBorderBreadth;
    nInfoBoxHeight = (nFontSlot * mScaledTotal) + nBorderBreadth;
    DrawInfoBox(c, mInfoTexture, x, y, nInfoBoxWidth, nInfoBoxHeight);

    // info text
    yText = y + nBorderBreadth;
    i = 0;
    for(i=0; i<mScaledTotal; i++)
    {
        // ammo info
        DoDrawText(c, mInfoFont, xText, yText, 238, 238, 0, mAmmoInfo[i].DamageType);
        // percentage info
        DoDrawText(c, mInfoFont, xPerc, yText, 238, 238, 0, mAmmoInfo[i].Percentage$"%");
        yText += nFontSlot;
    }

    // bottom info border
    y += nInfoBoxHeight;
    DrawBorder(c, x, y, nInfoBoxWidth, nBorderBreadth);

    // footer box
    y += nBorderBreadth;
    DrawInfoBox(c, mFooterTexture, x, y, nInfoBoxWidth, nHeaderBoxHeight);

    // footer text
    yText = y + nBorderBreadth;
    DoDrawText(c, mFooterFont, xText, yText, 245, 255, 250, sNote);

    // bottom border
    y += nHeaderBoxHeight;
    DrawBorder(c, x, y, nInfoBoxWidth, nBorderBreadth);

    nTotalHeight = (nBorderBreadth * 4) + (nHeaderBoxHeight*2) + nInfoBoxHeight;

    // left border
    x = nLeftEdge;
    y = nTopEdge;
    DrawBorder(c, x, y, nBorderBreadth, nTotalHeight);

    // right border
    x = nLeftEdge + nBorderBreadth + nInfoBoxWidth;
    y = nTopEdge;
    DrawBorder(c, x, y, nBorderBreadth, nTotalHeight);


    // You must do this for other mutators down the chain who want to draw on the canvas.
    if ( NextHUDMutator != None )
        NextHUDMutator.PostRender(c);
}

/*  If we come across ammo that has a damage scaling other than one,
    we will keep track of it, and figure out a percentage for it to
    present to the player rather than the damage scaling value.
*/
simulated function ProcessAmmo(float ADamageScaling, string ADamageType)
{
    local int i;
    local int nSpacesRequired;
    local int nPercentage;
    local float fPercentage;
    local float fOne;
    local float fHundred;
    local float fNegative;

    if(ADamageScaling == 1.0)
        return;

    fOne = 1.0;
    fHundred = 100.0;

    if(ADamageScaling > 1.0)
    {
        fPercentage = (ADamageScaling * fHundred) - fHundred;
    }
    else
    {
        fNegative = -1;
        fPercentage = (fHundred - (ADamageScaling * fHundred)) * fNegative;
    }

    // This rounds for us :)
    nPercentage = fPercentage;

    mAmmoInfo[mScaledTotal].DamageType = ADamageType;
    mAmmoInfo[mScaledTotal].Percentage = string(nPercentage);

    // Lame way to get text the same 'length', so they appear to right-align.
    i = 0;
    nSpacesRequired = (7 - len(mAmmoInfo[mScaledTotal].Percentage));
    while(i < nSpacesRequired)
    {
        // Add 2 spaces to make up for the width of a digit.
        mAmmoInfo[mScaledTotal].Percentage = "  "$mAmmoInfo[mScaledTotal].Percentage;
        i++;
    }

    mScaledTotal++;
}

simulated function DoDrawText(Canvas c, font AFont, int x, int y, int ARed, int AGreen, int ABlue, string AText)
{
    c.Font = AFont;
    c.DrawColor.R = ARed;
    c.DrawColor.G = AGreen;
    c.DrawColor.B = ABlue;
    c.SetPos(x, y);
    c.Style = ERenderStyle.STY_Normal;
    c.DrawText(AText, false);
}

simulated function DrawBorder(Canvas c, int x, int y, int AWidth, int AHeight)
{
    c.Style = ERenderStyle.STY_Modulated;
    c.SetPos(x, y);
    c.DrawTileClipped(mBorderTexture, AWidth, AHeight, 0, 0, 4, 4);
}

// Background box
simulated function DrawInfoBox(Canvas c, Texture aTexture, int x, int y, int AWidth, int AHeight)
{
    c.Style = ERenderStyle.STY_Modulated;
    c.SetPos(x, y);
    c.DrawRect(aTexture, AWidth, AHeight);
}

simulated function DrawInfoBoxB(Canvas c, Texture aTexture, int x, int y, int AWidth, int AHeight, int xClipStart, int yClipStart, int xClipWidth, int yClipWidth)
{
    c.Style = ERenderStyle.STY_Modulated;
    c.SetPos(x, y);
    c.DrawTileClipped(aTexture, AWidth, AHeight, xClipStart, yClipStart, xClipWidth, yClipWidth);
}

/*
    DrawRect
    p1 = texture
    p2 = width
    p3 = height
    
    DrawTileClipped
    p1 = texture
    p2 = width
    p3 = height
    for the next 4 parameters, you are talking about the position and size of the portion you want to 'cut out'.
    Once this piece it 'cut out', it will get scaled to fill in the size you specified with width and height.
    p4 = starting x point
    p5 = starting y point
    p6 = how many pixels to grab sideways from the x in p4
    p6 = how many pixels to grab down from the y in p5
*/

/*  Note for bNetTemporary: most of our variables are replicated when they get assigned 
    back in medWeaponAmplifier.PreBeginPlay(). However there is one variable, bGameOn,
    that gets assigned later. So we must have bNetTemporary=False so it will be replicated.
*/
defaultproperties
{
     RemoteRole=ROLE_SimulatedProxy
     bAlwaysRelevant=True
     bNetTemporary=False  // Set to false if you need to replicate data to the clients more than once per level
}
