//====================================================================================
// medWeaponAmplifier
// creator:	Helen! www.birdieman.com/forum
// created:	2010.08
// comment: Allows you to scale the damage of weapons/ammo
//====================================================================================
class medWeaponAmplifier expands mutator config(medWeaponAmplifier);

enum ELogType
{
	LOG_info,   // 0
	LOG_error   // 1
};

// Config vars
var() config bool bLogInfo;
var() config bool bDisplayHUD;
var() config bool bDeferToOtherDamageScaling;

var() config float chainsaw;
var() config float hammer;
var() config float pulse;
var() config float pulseAlt;
var() config float flakripper;
var() config float flakAlt;
var() config float ripperAlt;
var() config float rocket;
var() config float rocketAlt;
var() config float shock;
var() config float bio;
var() config float redeemer;
var() config float bullet;
var() config float selfInflicted;
var() config float falling;

var   config string mMap[256];

// class vars
var medWAHUD myWAHUD;
var bool bCheckForGameOn;

function PreBeginPlay()
{
    local int i;
    local bool bFound;
    local string aCurrentMap;

    if(!Level.Game.IsA('DeathMatchPlus'))
    {
        doLog(LOG_info, "PreBeginPlay() - Level.Game = "$string(Level.Game.Class)$", this mutator only supports games deriving from the DeathMatchPlus class");
        Disable('Tick');
        return;
    }

    // Defaults
    ValidateDamageScaling(chainsaw);
    ValidateDamageScaling(hammer);
    ValidateDamageScaling(pulse);
    ValidateDamageScaling(pulseAlt);
    ValidateDamageScaling(flakripper);
    ValidateDamageScaling(flakAlt);
    ValidateDamageScaling(ripperAlt);
    ValidateDamageScaling(rocket);
    ValidateDamageScaling(rocketAlt);
    ValidateDamageScaling(shock);
    ValidateDamageScaling(bio);
    ValidateDamageScaling(redeemer);
    ValidateDamageScaling(bullet);
    ValidateDamageScaling(selfInflicted);
    ValidateDamageScaling(falling);

    aCurrentMap = GetCurrentMap();

    if((aCurrentMap != "") && (ArrayCount(mMap) >= 0))
	{
	    // See if we have to implement some map specific values

        i = 0;
        bFound = false;
        while ((i < ArrayCount(mMap)) && (!bFound))
        {
            bFound = (aCurrentMap ~= GetSetting(mMap[i], "mapname"));

            if(bFound)
            {
                SetDamageScale(chainsaw, GetSetting(mMap[i], "chainsaw"));
                SetDamageScale(hammer, GetSetting(mMap[i], "hammer"));
                SetDamageScale(pulse, GetSetting(mMap[i], "pulse"));
                SetDamageScale(pulseAlt, GetSetting(mMap[i], "pulseAlt"));
                SetDamageScale(flakripper, GetSetting(mMap[i], "flakripper"));
                SetDamageScale(flakAlt, GetSetting(mMap[i], "flakAlt"));
                SetDamageScale(ripperAlt, GetSetting(mMap[i], "ripperAlt"));
                SetDamageScale(rocket, GetSetting(mMap[i], "rocket"));
                SetDamageScale(rocketAlt, GetSetting(mMap[i], "rocketAlt"));
                SetDamageScale(shock, GetSetting(mMap[i], "shock"));
                SetDamageScale(bio, GetSetting(mMap[i], "bio"));
                SetDamageScale(redeemer, GetSetting(mMap[i], "redeemer"));
                SetDamageScale(bullet, GetSetting(mMap[i], "bullet"));
                SetDamageScale(selfInflicted, GetSetting(mMap[i], "selfInflicted"));
                SetDamageScale(falling, GetSetting(mMap[i], "falling"));
            }
            else
            {
                i++;
            }
        } // while
    }

    Level.Game.RegisterDamageMutator(Self);

    if(bDisplayHUD)
    {
    	myWAHUD = Spawn(class 'medWAHUD');

        if(myWAHUD == None)
        {
            doLog(LOG_info, "Failed to spawn the HUD for unknown reasons :(");
            Disable('Tick');
            return;
        }

        myWAHUD.chainsaw = chainsaw;
        myWAHUD.hammer = hammer;
        myWAHUD.pulse = pulse;
        myWAHUD.pulseAlt = pulseAlt;
        myWAHUD.flakripper = flakripper;
        myWAHUD.flakAlt = flakAlt;
        myWAHUD.ripperAlt = ripperAlt;
        myWAHUD.rocket = rocket;
        myWAHUD.rocketAlt = rocketAlt;
        myWAHUD.shock = shock;
        myWAHUD.bio = bio;
        myWAHUD.redeemer = redeemer;
        myWAHUD.bullet = bullet;
        myWAHUD.selfInflicted = selfInflicted;
        myWAHUD.falling = falling;

        // It is now our job to check for when the game starts, because the medWAHUD class can't access this info.
        // Once we detect that the game has started, we will inform our myWAHUD.
        bCheckForGameOn = true;

        // We were telling the display not to draw until we set up the variables, but it is ready now, so...
    	myWAHUD.bCanRender = true;
    }
}

// Notice I did not put simulated in front. I don't need this to run on the client.
function Tick(float DeltaTime)
{
    if((!bCheckForGameOn) || (myWAHUD == None) || (Level.Game == None))
        return;

    // This is how I determine if the game has started. There wasn't a handy "bGameOn" flag,
    // so I looked in the DeathMatchPlus class for stuff I could use, and came up with this.
    if (!(DeathMatchPlus(Level.Game).bRequireReady && (DeathMatchPlus(Level.Game).Countdown > 0)))
    {
        Disable('Tick');
        myWAHUD.bGameOn = true;
    }
}

function SetDamageScale(out float ADamageScale, string sDamageValue)
{
    local float fValue;

    if(sDamageValue == "")
        return;

    // If sDamageValue is not a valid float, the result of this cast will be zero.
    fValue = float(sDamageValue);

    ValidateDamageScaling(fValue);

    ADamageScale = fValue;
}

function MutatorTakeDamage( out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, out Vector Momentum, name DamageType)
{
    local int nPreActualDamage;
    local bool bCanApplyScaling;
    local float fDamageScaling;
    local string sDamageType;

    doLog(LOG_info, "MutatorTakeDamage() - DamageType="$string(DamageType)$", ActualDamage="$string(ActualDamage));

    /*
    These are the values we will see coming in for the DamageType parameter.

    DamageType      our variable(s)     comment
    ==========      ===============     ==========
    slashed         chainsaw            chainsaw primary and alternate fire
    impact          hammer              hammer primary and alternate fire
    Pulsed          pulse               pulse gun primary fire
    zapped          pulseAlt            pulse gun alternate fire
    shredded        flakripper          primary fire for flak and ripper
    FlakDeath       flakAlt             flak alternate fire
    RipperAltDeath  ripperAlt           ripper alternate fire
    RocketDeath     rocket              rocket primary fire
    GrenadeDeath    rocketAlt           rocket alternate fire
    jolted          shock               shock rifle primary and alternate fire
    Corroded        bio                 bio primary and alternate fire
    RedeemerDeath   redeemer            redeemer primary and alternate fire
    shot            bullet              primary and alternate fire for enforcer, minigun, and sniper rifle
    Fell            falling             damage taken when the player falls too far
    [n/a]           selfInflicted       damage done by the player to themself


    Decapitated     (none)              Decapitated usually means dead, so we're not going to process this one.
    */

    // See if we have been told to defer to other damage scaling...
    bCanApplyScaling = ( (InstigatedBy == None) || (InstigatedBy.DamageScaling == 1.0) || (!bDeferToOtherDamageScaling) );
    if(!bCanApplyScaling)
        doLog(LOG_info, "MutatorTakeDamage() - The damage scaling will not be applied. InstigatedBy.DamageScaling="$string(InstigatedBy.DamageScaling)$" and we were told to defer to it");

    if( /* Again notice we accept the possibility that ActualDamage may be LESS than zero.
           Because hey, if some other mutator is having fun, we don't want to get in it's way. */
        (ActualDamage != 0) &&
        /* safety check for DamageType */
        (DamageType != 'Unspecified') &&
        /* We are allowed to apply our scaling, see above where this is set. */
        ( bCanApplyScaling )   )
    {
        // Keep track of damage before we possibly change it.
        nPreActualDamage = ActualDamage;
        
        // Initialize
        fDamageScaling = 1.0;
        sDamageType = "";

        if((InstigatedBy != None) && (Victim != None) && (InstigatedBy == Victim) && (selfInflicted != 1.0))
        {
            fDamageScaling = selfInflicted;
            sDamageType = "selfInflicted";
        }
        else if(DamageType == 'slashed')
            fDamageScaling = chainsaw;
        else if(DamageType == 'impact')
            fDamageScaling = hammer;
        else if(DamageType == 'Pulsed')
            fDamageScaling = pulse;
        else if(DamageType == 'zapped')
            fDamageScaling = pulseAlt;
        else if(DamageType == 'shredded')
            fDamageScaling = flakripper;
        else if(DamageType == 'FlakDeath')
            fDamageScaling = flakAlt;
        else if(DamageType == 'RipperAltDeath')
            fDamageScaling = ripperAlt;
        else if(DamageType == 'RocketDeath')
            fDamageScaling = rocket;
        else if(DamageType == 'GrenadeDeath')
            fDamageScaling = rocketAlt;
        else if(DamageType == 'jolted')
            fDamageScaling = shock;
        else if(DamageType == 'Corroded')
            fDamageScaling = bio;
        else if(DamageType == 'RedeemerDeath')
            fDamageScaling = redeemer;
        else if(DamageType == 'shot')
            fDamageScaling = bullet;
        else if(DamageType == 'Fell')
            fDamageScaling = falling;

        if(fDamageScaling != 1.0)
        {
            ActualDamage *= fDamageScaling;
            
            if(sDamageType == "")
                sDamageType = string(DamageType);

            doLog(LOG_info, "MutatorTakeDamage() - "$sDamageType$" damage has been scaled! DamageScaling="$string(fDamageScaling)$", original Damage="$string(nPreActualDamage)$", new Damage="$string(ActualDamage));
        }
    }

	if ( NextDamageMutator != None )
		NextDamageMutator.MutatorTakeDamage( ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType );
}

function string GetCurrentMap()
{
    // Gets the name of the map being played.

    local string s;
    local int i;

    s = string(Level.Game.BaseMutator);

    if (s == "")
        return "";

    doLog(LOG_info, "Level.Game.BaseMutator = "$s);

    i = InStr(s, ".");
    if(i != (-1))
    {
        // Parse out the map name
        s = Left(s, i);
    }

    if(s!="")
        doLog(LOG_info, "Map Name = "$s);

    return s;
}

function ValidateDamageScaling(out float ADamageScaling)
{
    // We will treat a damage scaling value of zero as invalid. This covers both
    // when the user has specifically set it to zero, and also when they have
    // omitted to provide a value. Notice we allow a value less than zero, 
    // because if the user wants to be crazy with the settings, so be it.
    // But we WILL prevent ridiculous values.
    if(ADamageScaling == 0)
        ADamageScaling = 1;
    else if(ADamageScaling > 999)
        ADamageScaling = 999;
    else if(ADamageScaling < (-999))
        ADamageScaling = (-999);
}

function string GetSetting(string ASource, string ASetting)
{
    local int i;
    local string s;
    local bool bFound;

    if (ASource == "")
        return "";

    ASource = myReplace(ASource, " ", "");
    ASource = Caps(ASource);
    ASetting = Caps(ASetting);

    bFound = false;
    do
    {
        i = InStr(ASource, ",");
        if(i == (-1))
        {
            s = ASource;
            ASource = "";
        }
        else
        {
            s = Left(ASource, i);
            ASource = Mid(ASource, i+1);
        }

        // Sometimes the MAIN setting of the string does not have a qualifer (setting name + equal sign),
        // rather it is by itself. Check for these first.
        //  - mapname
        //  - texturename
        bFound = ( (InStr(s, "=") == -1) && ((ASetting ~= "mapname") || (ASetting ~= "texturename")) );

        if (!bFound)
        {
            // check for other settings
            i = InStr(s, ASetting$"=");
            if(i != -1)
            {
                // Note we have to skip over entire length of setting
                s = Mid(s, i+Len(ASetting$"="));
                bFound = (s != "");
            }
        }

    } until ((bFound) || (ASource == ""));

    if(bFound)
        return s;
    else
        return "";
}

function string myReplace(string ASource, string AReplace, string AWith)
{
	local int i;
	local string aResult;

	i = InStr(ASource, AReplace);
	
	if(i==-1)
	   return ASource;
	  
    aResult = "";

	do
	{
		aResult = aResult $ Left(ASource, i) $ AWith;
		ASource = Mid(ASource, i + Len(AReplace));
		i = InStr(ASource, AReplace);
	} until (i==-1);

    if(ASource != "")
        aResult = aResult $ ASource;

	return aResult;
}

function doLog(ELogType ALogType, string AMessage)
{
    if( (ALogType == LOG_info) && (bLogInfo) )
        log(AMessage, 'medWeaponAmplifier');
}

defaultproperties
{
    bAlwaysRelevant=true
}

